import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

import java.awt.Color;
import java.util.List;

/**
 * A ball - these are the main objects in the game.
 * 
 * @author Michael Berry (mjrb4) 
 * @version 27/12/09
 */
public class Ball extends Actor
{
    public static final int SPEED = 5;
    public static final GreenfootImage ballImage = new GreenfootImage("ball.png");
    public static final Color selectedColor = Color.PINK;
    private static Ball selected;
    private static boolean ready;
    private Vector velocity;
    
    /**
     * Create a new ball.
     */
    public Ball() {
        ready = true;
        velocity = new Vector(0, 0);
    }
    
    /**
     * Set the speed of this ball to match a certain vector.
     */
    public void setSpeed(Vector v) {
        velocity.setX(v.getX());
        velocity.setY(v.getY());
    }
    
    /**
     * If we're selected and arrow keys are pressed, move until we hit another ball.
     */
    public void act() {
        if(selected!=this) setImage(ballImage);
        if(selected==this && ready) {
            doKeys();
        }
        if(ready) {
            checkSelected();
        }
        move();
        checkCollision();
        
        //This check must be last
        checkOutOfWorld();
    }
    
    /**
     * Manage the input.
     */
    private void doKeys() {
        if(Greenfoot.isKeyDown("up") && Greenfoot.isKeyDown("up")) {
            velocity.setY(-1*SPEED);
            ready = false;
        }
        else if(Greenfoot.isKeyDown("down") && Greenfoot.isKeyDown("down")) {
            velocity.setY(SPEED);
            ready = false;
        }
        else if(Greenfoot.isKeyDown("left") && Greenfoot.isKeyDown("left")) {
            velocity.setX(-1*SPEED);
            ready = false;
        }
        else if(Greenfoot.isKeyDown("right") && Greenfoot.isKeyDown("right")) {
            velocity.setX(SPEED);
            ready = false;
        }
    }
    
    /**
     * Check collisions with any neighbouring balls, and if one is found
     * stop and pass on the speed to the next ball.
     */
    private void checkCollision() {
        Ball ball = (Ball)getOneIntersectingObject(Ball.class);
        if(ball != null) {
            ball.setSpeed(velocity);
            velocity.setNeutral();
        }
    }
    
    /**
     * Check whether this ball has been selected (and update the
     * selected ball accordingly.)
     */
    private void checkSelected() {
        MouseInfo mi = Greenfoot.getMouseInfo();
        if(mi!=null && mi.getActor()==this && mi.getButton()==1) {
            if(selected != null) {
                selected.setImage(ballImage);
            }
            selected = this;
            setImage(new ColoredImage(getImage(), selectedColor));
        }
        else if(mi!=null && mi.getActor()==null && mi.getButton()==1) {
            selected = null;
        }
    }
    
    /**
     * Check whether a ball is out of the world, and remove it from the 
     * world if it is.
     */
    private void checkOutOfWorld() {
        if(getX()<SPEED || getX()>getWorld().getWidth()-SPEED || getY()<SPEED || getY()>getWorld().getHeight()-SPEED) {
            if(selected==this) {
                ((BallWorld)getWorld()).fail();
            }
            else {
                checkPass();
                if(getWorld()!=null) {
                    getWorld().removeObject(this);
                }
                ready = true;
            }
        }
    }
    
    /**
     * Check whether this level has been passed.
     */
    private void checkPass() {
        List<Ball> actors = getWorld().getObjects(Ball.class);
        if(actors.size()==2) {
            ((BallWorld)getWorld()).pass();
        }
    }
    
    /**
     * Move according to the components set in the vector.
     */
    private void move() {
        int x = (int)(getX()+velocity.getX());
        int y = (int)(getY()+velocity.getY());
        setLocation(x, y);
    }
}
 