import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.Collection;

/**
 * The world where the game is played.
 * 
 * @author Michael Berry (mjrb4)
 * @version 27/12/09
 */
public class BallWorld  extends World
{

    private int level = 1;
    private LevelText levelText;
    private ScoreText scoreText;
    private TimerText timerText;

    /**
     * Create a new world
     */
    public BallWorld() {
        super(500, 500, 1);
        setPaintOrder(SmallText.class, Ball.class, BigText.class);
        scoreText = new ScoreText();
        levelText = new LevelText();
        timerText = new TimerText();
        addObject(levelText, 490, 15);
        addObject(scoreText, 490, 30);
        addObject(timerText, 490, 45);
        createLevel(level);
        Greenfoot.start();
    }
    
    /**
     * Create a particular level. This just consists
     * of placing a ball in the world.
     */
    public void createLevel(int level) {
        clearBalls();
        if(level==1) {
            addObject(new Ball(), getWidth()/2, 100);
            addObject(new Ball(), getWidth()/2, 200);
        }
        else if(level==2) {
            addObject(new Ball(), getWidth()/2, 100);
            addObject(new Ball(), getWidth()/2, 200);
            addObject(new Ball(), 450, 170);
        }
        else if(level==3) {
            addObject(new Ball(), 250, 100);
            addObject(new Ball(), 250, 200);
            addObject(new Ball(), 390, 180);
            addObject(new Ball(), 280, 390);
        }
        else if(level==4) {
            addObject(new Ball(), 220, 90);
            addObject(new Ball(), 250, 200);
            addObject(new Ball(), 390, 130);
            addObject(new Ball(), 100, 130);
            addObject(new Ball(), 130, 240);
        }
        else if(level==5) {
            addObject(new Ball(), 460, 200);
            addObject(new Ball(), 270, 200);
            addObject(new Ball(), 290, 80);
            addObject(new Ball(), 290, 420);
            addObject(new Ball(), 170, 170);
            addObject(new Ball(), 200, 420);

        }
        else {
            int score = scoreText.getScore();
            Collection<Actor> actors = getObjects(null);
            for(Actor actor : actors) {
                removeObject(actor);
            }
            BigText text = new BigText();
            addObject(text, getWidth()/2-50, getHeight()/2);
            text.setText("Final score: " + score);
            Greenfoot.stop();
        }
    }
    
    /**
     * Not used in the game - helper method for generating the code
     * I need for each level by just dragging the balls in the world.
     * Easier than figuring it out by hand!
     */
    public void generateLevelCode() {
        Collection<Ball> balls = getObjects(Ball.class);
        for(Ball ball : balls) {
            int x = ((ball.getX()+5)/10)*10;
            int y = ((ball.getY()+5)/10)*10;
            System.out.println("addObject(new Ball(), " + x + ", " + y + ");");
        }
    }
    
    /**
     * Called if the player has failed a particular level.
     */
    public void fail() {
        BigText text = new BigText();
        addObject(text, getWidth()/2, getHeight()/2);
        text.setText("Failed!");
        createLevel(level);
    }
    
    /**
     * Called if the player has passed a particular level.
     */
    public void pass() {
        level++;
        levelText.advanceLevel();
        scoreText.addScore(300/timerText.getTime());
        timerText.reset();
        BigText text = new BigText();
        addObject(text, getWidth()/2, getHeight()/2);
        text.setText("Level complete!");
        createLevel(level);
    }
    
    /**
     * Removes all balls from the world.
     */
    private void clearBalls() {
        Collection<Ball> balls = getObjects(Ball.class);
        for(Ball ball : balls) {
            removeObject(ball);
        }
    }
}
